# -*- coding: utf-8 -*-
import socket, logging, sys, time
from handler import Handler
from collection import QueueCollection

DEFAULT_HOST = '127.0.0.1'
DEFAULT_PORT = 21122
DEFAULT_PATH = '/tmp/peafowl/'
DEFAULT_TIMEOUT = 60
DEFAULT_PID = '/var/run/peafowl.pid'
DEFAULT_VERBOSITY = 30

class Server(object):
    def __init__(self, **kwargs):
        """
        Initialize a new Peafowl server, but do not accept connections or
        process requests.
        """
        opts = {'host':DEFAULT_HOST, 'port':DEFAULT_PORT, 'path':DEFAULT_PATH, 'timeout':DEFAULT_TIMEOUT, 'debug':DEFAULT_VERBOSITY}
        opts.update(kwargs)
        if opts.has_key('log'):            
            logging.basicConfig(filename=opts['log'], level=DEFAULT_VERBOSITY - opts['debug'], format='%(asctime)s %(levelname)s %(message)s')
        else:
            logging.basicConfig(level=DEFAULT_VERBOSITY - opts['debug'], format='%(asctime)s %(levelname)s %(message)s')
        self.queue_collection = QueueCollection(opts['path'])
        self.stats = {'bytes_read':0, 'bytes_written':0, 'start_time':time.time(), 'connections':0, 
                      'total_connections':0, 'get_requests':0, 'set_requests':0}
        self.stats['start_time'] = time.time()
        self._bind(opts['host'], opts['port'])
    
    def _bind(self, host, port):
        try:
            self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.server.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            self.server.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
            self.server.bind((host, port))
            self.server.listen(5)
            logging.info("Listening to %s on port %s" % (host, port))
        except socket.error, e:
            if self.server:
                self.server.close()
            logging.error("Could not open socket: %s" % e.message)
            sys.exit(1)
    
    def run(self):
        while True:
            try:
                self.stats['connections'] += 1
                client, address = self.server.accept()
                Handler(client, self.queue_collection, self.stats).start()
                self.stats['connections'] -= 1
            except socket.error, e:
                sys.exit(1)
    
    def stop(self):
        self.queue_collection.close()
        if self.server:
            self.server.close()
    
    @staticmethod
    def start(**kwargs):
        """
        Start listening and processing requests.
        """
        server = Server(**kwargs)
        server.run()
        return server




# -*- coding: utf-8 -*-
import unittest, random, time, hashlib, os
from peafowl.collection import QueueCollection, QueueCollectionError
try:
    from cmemcache import Client
except ImportError:
    from memcache import Client

class TestPeafowl(unittest.TestCase):
    def setUp(self):
        self.memcache = Client(['127.0.0.1:21122'])
        
    def tearDown(self):
        self.memcache.disconnect_all()

    def test_set_and_get_one_entry(self):
        v = random.randint(1, 32)
        self.assertEqual(None, self.memcache.get('test_set_and_get_one_entry'))
        self.memcache.set('test_set_and_get_one_entry', v)
        self.assertEqual(v, self.memcache.get('test_set_and_get_one_entry'))
    
    def test_set_with_expiry(self):
        v = random.randint(1, 32)
        self.assertEqual(None, self.memcache.get('test_set_with_expiry'))
        now = time.time()
        self.memcache.set('test_set_with_expiry', v + 2, now)
        self.memcache.set('test_set_with_expiry', v)
        time.sleep(now + 1 - time.time())
        self.assertEqual(v, self.memcache.get('test_set_with_expiry'))

    def test_long_value(self):
        string = ''.join([random.choice("abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ23456789") for i in range(300)])
        self.memcache.set('test_long_value', string)
        string = self.memcache.get('test_long_value')
        self.assertEqual(len(string), 300)

    def test_stats(self):
        (key, stats) = self.memcache.get_stats()[0]
        self.assertEqual('127.0.0.1:21122 (1)', key)
        keys = ['bytes', 'pid', 'time', 'limit_maxbytes', 
                 'cmd_get', 'version', 'bytes_written', 'cmd_set', 
                 'get_misses', 'total_connections', 'curr_connections', 
                 'curr_items', 'uptime', 'get_hits', 'total_items', 
                 'rusage_system', 'rusage_user', 'bytes_read']
        for key in keys:
            self.assert_(stats.has_key(key), "key '%s' is not in stats" % key)
    
    def test_unknown_command_returns_valid_result(self):
        response = self.memcache.add('blah', 1)
        self.assertEqual(False, response)

    def test_that_disconnecting_and_reconnecting_works(self):
        v = random.randint(1, 32)
        self.memcache.set('test_that_disconnecting_and_reconnecting_works', v)
        self.memcache.disconnect_all()
        self.assertEqual(v, int(self.memcache.get('test_that_disconnecting_and_reconnecting_works')))

    def test_creating_queue_collection_with_invalid_path_throws__exception(self):
        invalid_path = None
        while invalid_path == None or os.path.exists(invalid_path):
            v = str(random.randint(1, 32))
            invalid_path = os.path.join('/', hashlib.md5(v).hexdigest())
        try:
            collection = QueueCollection(invalid_path)
            self.fail("QueueCollectionError has not been raised")
        except QueueCollectionError:
            pass

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
__version__ = "0.7"
from server import Server
from runner import Runner
from handler import Handler
from queue import PersistentQueue
from collection import QueueCollection
 
__all__ = [Server, Runner, Handler, PersistentQueue, QueueCollection]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
import peafowl

setup(name='peafowl',
      version=peafowl.__version__,
      description='A light weight server for reliable distributed message passing.',
      long_description="Peafowl is a powerful but simple messaging server that enables reliable distributed queuing with an absolutely minimal overhead. It speaks the MemCache protocol for maximum cross-platform compatibility. Any language that speaks MemCache can take advantage of Peafowl's queue facilities.",
      author='Timothee Peignier',
      author_email='tim@tryphon.org',
      url='http://code.google.com/p/peafowl/',
      license = 'MIT License',
      platforms = ["Unix",],
      keywords = "peafowl queue messaging distributed memcache starling",
      classifiers = [ "Development Status :: 4 - Beta",
                      "License :: OSI Approved :: MIT License",
                      "Operating System :: Unix",
                      "Programming Language :: Python" ],
      packages = ['peafowl'],
      scripts=['bin/peafowl'],
      test_suite='tests'
)
